@@ -1,16 +1,24 @@
 /*
+ * $PostgreSQL: pgsql/contrib/spi/timetravel.c,v 1.31 2009/06/11 14:48:52 momjian Exp $
+ *
+ *
  * timetravel.c --	function to get time travel feature
  *		using general triggers.
+ *
+ * Modified by BJTHE Zoltn, Hungary, mailto:urdesobt@axelero.hu
  */
+#include "postgres.h"
 
-/* Modified by BJTHE Zoltn, Hungary, mailto:urdesobt@axelero.hu */
+#include <ctype.h>
 
-#include "executor/spi.h"		/* this is what you need to work with SPI */
-#include "commands/trigger.h"	/* -"- and triggers */
-#include "miscadmin.h"			/* for GetPgUserName() */
-#include <ctype.h>				/* tolower () */
+#include "catalog/pg_type.h"
+#include "commands/trigger.h"
+#include "executor/spi.h"
+#include "miscadmin.h"
+#include "utils/builtins.h"
+#include "utils/nabstime.h"
 
-#define ABSTIMEOID	702			/* it should be in pg_type.h */
+PG_MODULE_MAGIC;
 
 /* AbsoluteTime currabstime(void); */
 Datum		timetravel(PG_FUNCTION_ARGS);
@@ -20,8 +28,8 @@ Datum		get_timetravel(PG_FUNCTION_ARGS);
 typedef struct
 {
 	char	   *ident;
-	void	   *splan;
-}	EPlan;
+	SPIPlanPtr	splan;
+} EPlan;
 
 static EPlan *Plans = NULL;		/* for UPDATE/DELETE */
 static int	nPlans = 0;
@@ -30,26 +38,26 @@ typedef struct _TTOffList
 {
 	struct _TTOffList *next;
 	char		name[1];
-}	TTOffList;
+} TTOffList;
 
 static TTOffList TTOff = {NULL, {0}};
 
 static int	findTTStatus(char *name);
-static EPlan *find_plan(char *ident, EPlan ** eplan, int *nplans);
+static EPlan *find_plan(char *ident, EPlan **eplan, int *nplans);
 
 /*
  * timetravel () --
- *		1.	IF an update affects tuple with stop_date eq INFINITY
+ *		1.  IF an update affects tuple with stop_date eq INFINITY
  *			then form (and return) new tuple with start_date eq current date
  *			and stop_date eq INFINITY [ and update_user eq current user ]
  *			and all other column values as in new tuple, and insert tuple
  *			with old data and stop_date eq current date
  *			ELSE - skip updation of tuple.
- *		2.	IF an delete affects tuple with stop_date eq INFINITY
+ *		2.  IF an delete affects tuple with stop_date eq INFINITY
  *			then insert the same tuple with stop_date eq current date
  *			[ and delete_user eq current user ]
  *			ELSE - skip deletion of tuple.
- *		3.	On INSERT, if start_date is NULL then current date will be
+ *		3.  On INSERT, if start_date is NULL then current date will be
  *			inserted, if stop_date is NULL then INFINITY will be inserted.
  *			[ and insert_user eq current user, update_user and delete_user
  *			eq NULL ]
@@ -111,7 +119,7 @@ timetravel(PG_FUNCTION_ARGS)
 
 	/* Should be called for ROW trigger */
 	if (TRIGGER_FIRED_FOR_STATEMENT(trigdata->tg_event))
-		elog(ERROR, "timetravel: can't process STATEMENT events");
+		elog(ERROR, "timetravel: cannot process STATEMENT events");
 
 	/* Should be called BEFORE */
 	if (TRIGGER_FIRED_AFTER(trigdata->tg_event))
@@ -168,7 +176,7 @@ timetravel(PG_FUNCTION_ARGS)
 	}
 
 	/* create fields containing name */
-	newuser = DirectFunctionCall1(textin, CStringGetDatum(GetUserNameFromId(GetUserId())));
+	newuser = CStringGetTextDatum(GetUserNameFromId(GetUserId()));
 
 	nulltext = (Datum) NULL;
 
@@ -243,8 +251,8 @@ timetravel(PG_FUNCTION_ARGS)
 		elog(ERROR, "timetravel (%s): %s must be NOT NULL", relname, args[a_time_off]);
 
 	/*
-	 * If DELETE/UPDATE of tuple with stop_date neq INFINITY then say
-	 * upper Executor to skip operation for this tuple
+	 * If DELETE/UPDATE of tuple with stop_date neq INFINITY then say upper
+	 * Executor to skip operation for this tuple
 	 */
 	if (newtuple != NULL)
 	{							/* UPDATE */
@@ -257,12 +265,11 @@ timetravel(PG_FUNCTION_ARGS)
 			elog(ERROR, "timetravel (%s): %s must be NOT NULL", relname, args[a_time_off]);
 
 		if (oldtimeon != newtimeon || oldtimeoff != newtimeoff)
-			elog(ERROR, "timetravel (%s): you can't change %s and/or %s columns (use set_timetravel)",
+			elog(ERROR, "timetravel (%s): you cannot change %s and/or %s columns (use set_timetravel)",
 				 relname, args[a_time_on], args[a_time_off]);
 	}
 	if (oldtimeoff != NOEND_ABSTIME)
-	{							/* current record is a deleted/updated
-								 * record */
+	{							/* current record is a deleted/updated record */
 		pfree(relname);
 		return PointerGetDatum(NULL);
 	}
@@ -283,8 +290,7 @@ timetravel(PG_FUNCTION_ARGS)
 	}
 
 	/* change date column(s) */
-	cvals[attnum[a_time_off] - 1] = newtimeoff; /* stop_date eq current
-												 * date */
+	cvals[attnum[a_time_off] - 1] = newtimeoff; /* stop_date eq current date */
 	cnulls[attnum[a_time_off] - 1] = ' ';
 
 	if (!newtuple)
@@ -297,8 +303,8 @@ timetravel(PG_FUNCTION_ARGS)
 	}
 
 	/*
-	 * Construct ident string as TriggerName $ TriggeredRelationId and try
-	 * to find prepared execution plan.
+	 * Construct ident string as TriggerName $ TriggeredRelationId and try to
+	 * find prepared execution plan.
 	 */
 	snprintf(ident, sizeof(ident), "%s$%u", trigger->tgname, rel->rd_id);
 	plan = find_plan(ident, &Plans, &nPlans);
@@ -306,7 +312,7 @@ timetravel(PG_FUNCTION_ARGS)
 	/* if there is no plan ... */
 	if (plan->splan == NULL)
 	{
-		void	   *pplan;
+		SPIPlanPtr	pplan;
 		Oid		   *ctypes;
 		char		sql[8192];
 		char		separ = ' ';
@@ -337,9 +343,8 @@ timetravel(PG_FUNCTION_ARGS)
 			elog(ERROR, "timetravel (%s): SPI_prepare returned %d", relname, SPI_result);
 
 		/*
-		 * Remember that SPI_prepare places plan in current memory context
-		 * - so, we have to save plan in Top memory context for latter
-		 * use.
+		 * Remember that SPI_prepare places plan in current memory context -
+		 * so, we have to save plan in Top memory context for latter use.
 		 */
 		pplan = SPI_saveplan(pplan);
 		if (pplan == NULL)
@@ -396,8 +401,8 @@ timetravel(PG_FUNCTION_ARGS)
 		rettuple = SPI_modifytuple(rel, newtuple, chnattrs, chattrs, newvals, newnulls);
 
 		/*
-		 * SPI_copytuple allocates tmptuple in upper executor context -
-		 * have to free allocation using SPI_pfree
+		 * SPI_copytuple allocates tmptuple in upper executor context - have
+		 * to free allocation using SPI_pfree
 		 */
 		/* SPI_pfree(tmptuple); */
 	}
@@ -512,7 +517,7 @@ currabstime()
 */
 
 static EPlan *
-find_plan(char *ident, EPlan ** eplan, int *nplans)
+find_plan(char *ident, EPlan **eplan, int *nplans)
 {
 	EPlan	   *newp;
 	int			i;
