@@ -183,7 +183,7 @@ assign_datestyle(const char *value, bool doit, GucSource source)
 		return value;
 
 	/*
-	 * Prepare the canonical string to return.	GUC wants it malloc'd.
+	 * Prepare the canonical string to return.  GUC wants it malloc'd.
 	 */
 	result = (char *) malloc(32);
 	if (!result)
@@ -241,6 +241,8 @@ assign_timezone(const char *value, bool doit, GucSource source)
 	char	   *result;
 	char	   *endptr;
 	double		hours;
+	int			new_ctimezone;
+	pg_tz	   *new_tz;
 
 	/*
 	 * Check for INTERVAL 'foo'
@@ -268,7 +270,7 @@ assign_timezone(const char *value, bool doit, GucSource source)
 
 		/*
 		 * Try to parse it.  XXX an invalid interval format will result in
-		 * ereport(ERROR), which is not desirable for GUC.	We did what we
+		 * ereport(ERROR), which is not desirable for GUC.  We did what we
 		 * could to guard against this in flatten_set_variable_args, but a
 		 * string coming in from postgresql.conf might contain anything.
 		 */
@@ -294,16 +296,28 @@ assign_timezone(const char *value, bool doit, GucSource source)
 			pfree(interval);
 			return NULL;
 		}
+
+		/* Here we change from SQL to Unix sign convention */
+#ifdef HAVE_INT64_TIMESTAMP
+		new_ctimezone = -(interval->time / USECS_PER_SEC);
+#else
+		new_ctimezone = -interval->time;
+#endif
+		new_tz = pg_tzset_offset(new_ctimezone);
+
+		if (!new_tz)
+		{
+			ereport(GUC_complaint_elevel(source),
+					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+					 errmsg("invalid interval value for time zone: out of range")));
+			pfree(interval);
+			return NULL;
+		}
+
 		if (doit)
 		{
-			/* Here we change from SQL to Unix sign convention */
-#ifdef HAVE_INT64_TIMESTAMP
-			CTimeZone = -(interval->time / USECS_PER_SEC);
-#else
-			CTimeZone = -interval->time;
-#endif
-			session_timezone = pg_tzset_offset(CTimeZone);
-
+			CTimeZone = new_ctimezone;
+			session_timezone = new_tz;
 			HasCTZSet = true;
 		}
 		pfree(interval);
@@ -316,11 +330,22 @@ assign_timezone(const char *value, bool doit, GucSource source)
 		hours = strtod(value, &endptr);
 		if (endptr != value && *endptr == '\0')
 		{
+			/* Here we change from SQL to Unix sign convention */
+			new_ctimezone = -hours * SECS_PER_HOUR;
+			new_tz = pg_tzset_offset(new_ctimezone);
+
+			if (!new_tz)
+			{
+				ereport(GUC_complaint_elevel(source),
+						(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+						 errmsg("invalid value for time zone: out of range")));
+				return NULL;
+			}
+
 			if (doit)
 			{
-				/* Here we change from SQL to Unix sign convention */
-				CTimeZone = -hours * SECS_PER_HOUR;
-				session_timezone = pg_tzset_offset(CTimeZone);
+				CTimeZone = new_ctimezone;
+				session_timezone = new_tz;
 				HasCTZSet = true;
 			}
 		}
@@ -335,7 +360,7 @@ assign_timezone(const char *value, bool doit, GucSource source)
 			 *
 			 * During GUC initialization, since the timezone library isn't set
 			 * up yet, pg_get_timezone_name will return NULL and we will leave
-			 * the setting as UNKNOWN.	If this isn't overridden from the
+			 * the setting as UNKNOWN.  If this isn't overridden from the
 			 * config file then pg_timezone_initialize() will eventually
 			 * select a default value from the environment.
 			 */
@@ -352,8 +377,6 @@ assign_timezone(const char *value, bool doit, GucSource source)
 			/*
 			 * Otherwise assume it is a timezone name, and try to load it.
 			 */
-			pg_tz	   *new_tz;
-
 			new_tz = pg_tzset(value);
 
 			if (!new_tz)
@@ -391,7 +414,7 @@ assign_timezone(const char *value, bool doit, GucSource source)
 		return value;
 
 	/*
-	 * Prepare the canonical string to return.	GUC wants it malloc'd.
+	 * Prepare the canonical string to return.  GUC wants it malloc'd.
 	 */
 	if (HasCTZSet)
 	{
@@ -467,7 +490,7 @@ assign_log_timezone(const char *value, bool doit, GucSource source)
 		 *
 		 * During GUC initialization, since the timezone library isn't set up
 		 * yet, pg_get_timezone_name will return NULL and we will leave the
-		 * setting as UNKNOWN.	If this isn't overridden from the config file
+		 * setting as UNKNOWN.  If this isn't overridden from the config file
 		 * then pg_timezone_initialize() will eventually select a default
 		 * value from the environment.
 		 */
@@ -521,7 +544,7 @@ assign_log_timezone(const char *value, bool doit, GucSource source)
 		return value;
 
 	/*
-	 * Prepare the canonical string to return.	GUC wants it malloc'd.
+	 * Prepare the canonical string to return.  GUC wants it malloc'd.
 	 */
 	result = strdup(value);
 
@@ -656,7 +679,7 @@ assign_client_encoding(const char *value, bool doit, GucSource source)
 
 	/*
 	 * Note: if we are in startup phase then SetClientEncoding may not be able
-	 * to really set the encoding.	In this case we will assume that the
+	 * to really set the encoding.  In this case we will assume that the
 	 * encoding is okay, and InitializeClientEncoding() will fix things once
 	 * initialization is complete.
 	 */
@@ -684,7 +707,7 @@ assign_client_encoding(const char *value, bool doit, GucSource source)
  * limit on names, so we can tell whether we're being passed an initial
  * role name or a saved/restored value.  (NOTE: we rely on guc.c to have
  * properly truncated any incoming value, but not to truncate already-stored
- * values.	See GUC_IS_NAME processing.)
+ * values.  See GUC_IS_NAME processing.)
  */
 extern char *session_authorization_string;		/* in guc.c */
 
