@@ -4,10 +4,10 @@
  *	  prototypes for tstoreReceiver.c
  *
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/include/executor/tstoreReceiver.h,v 1.7.6.1 2008/12/01 17:06:41 tgl Exp $
+ * $PostgreSQL: pgsql/src/include/executor/tstoreReceiver.h,v 1.15 2010/01/02 16:58:03 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,10 +19,11 @@
 #include "utils/tuplestore.h"
 
 
-extern DestReceiver *CreateTuplestoreDestReceiver(Tuplestorestate *tStore,
-							 MemoryContext tContext);
+extern DestReceiver *CreateTuplestoreDestReceiver(void);
 
-extern void SetTuplestoreDestReceiverDeToast(DestReceiver *self,
-											 bool detoast);
+extern void SetTuplestoreDestReceiverParams(DestReceiver *self,
+								Tuplestorestate *tStore,
+								MemoryContext tContext,
+								bool detoast);
 
 #endif   /* TSTORE_RECEIVER_H */
