@@ -4,10 +4,10 @@
  *	  Definitions for network authentication routines
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: auth.h,v 1.20 2001/11/05 17:46:33 momjian Exp $
+ * $PostgreSQL: pgsql/src/include/libpq/auth.h,v 1.37 2010/01/02 16:58:04 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -16,16 +16,12 @@
 
 #include "libpq/libpq-be.h"
 
-/*----------------------------------------------------------------
- * Common routines and definitions
- *----------------------------------------------------------------
- */
-
-void		ClientAuthentication(Port *port);
-
-#define PG_KRB4_VERSION "PGVER4.1"		/* at most KRB_SENDAUTH_VLEN chars */
-#define PG_KRB5_VERSION "PGVER5.1"
-
 extern char *pg_krb_server_keyfile;
+extern char *pg_krb_srvnam;
+extern bool pg_krb_caseins_users;
+extern char *pg_krb_server_hostname;
+extern char *pg_krb_realm;
+
+extern void ClientAuthentication(Port *port);
 
 #endif   /* AUTH_H */
