@@ -38,7 +38,3 @@ CREATE TABLE xsequences (
 	sequence text not null
 	) ;
 CREATE INDEX xsequences_pid_idx ON xsequences(pid) ;
-
-CREATE TABLE unicode_test (
-	testvalue  text NOT NULL
-);
